package nu.zoom.catonine;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.nio.charset.Charset;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Tailer that tracks which file to tail and which char.set to use to convert
 * the file contents to strings.
 * 
 */
public abstract class AbstractFileTailer extends
		AbstractRegularExpressionLogBlockTailer {
	private final Log log = LogFactory.getLog(getClass());
	private File file = null;
	private Charset charset = Charset.defaultCharset();

	/**
	 * Create a file tailer
	 * 
	 * @param file
	 *            The file to tail. May be null.
	 * @param charset
	 *            The charset to user, if null the platform default charset will
	 *            be used.
	 * @param sleepTime
	 *            {@link AbstractRegularExpressionLogBlockTailer}
	 */
	public AbstractFileTailer(File file, Charset charset, long sleepTime) {
		super(sleepTime);
		this.file = file;
		if (charset != null) {
			log.trace("Charset: " + charset);
			this.charset = charset;
		}
	}

	/**
	 * Set the file to follow.
	 * 
	 * @param file
	 *            The file to follow, may not be null. Must exist and be
	 *            readable.
	 * @throws IllegalArgumentException
	 *             If the file is null.
	 * @throws FileNotFoundException
	 *             If the file can not be found.
	 * @throws IOException
	 *             If already tailing a file the previous file could not be
	 *             closed.
	 */
	public synchronized void setFile(File file)
			throws IllegalArgumentException, FileNotFoundException, IOException {
		if (file == null) {
			throw new IllegalArgumentException("file may not be null");
		}
		stop();
		log.trace("Setting file to: " + file);
		this.file = file;
		fileToTailChanged(file);
	}
        
        public synchronized File getFile() throws IOException {
            return new File(file.getCanonicalPath()) ;
        }

	/**
	 * Notofies subclasses that the file to tail has changed.
	 * 
	 * @param newfile
	 *            The new file to tail.
	 * @throws IOException
	 *             If the subclass encounters io errors when processing the new
	 *             file.
	 */
	protected abstract void fileToTailChanged(File newfile) throws IOException;

	/**
	 * Set the character set. Can be used to decode the file.
	 * 
	 * @param charsetName
	 *            The name of the chracter set. If null or not supported the
	 *            default character set will be used instead, i.e. this class
	 *            will silently use the default.
	 */
	public synchronized void setCharSet(String charsetName) {
		if ((charsetName == null) || (!Charset.isSupported(charsetName))) {
			log.warn("Using default charset. Charset named: " + charsetName
					+ " is not supported");
			charset = Charset.defaultCharset();
		} else {
			charset = Charset.forName(charsetName);
		}
		log.trace("Charset changed to: " + charset);
		charsetChanged(charset);
	}

	/**
	 * Get the charset this class remembers.
	 * 
	 * @return The current character set.
	 */
	public synchronized Charset getCharset() {
		return charset;
	}

	/**
	 * Inform subclasses that the character set has changed.
	 * 
	 * @param newCharset
	 */
	protected abstract void charsetChanged(Charset newCharset);

	/**
	 * Validates that this instance is in a valid state.
	 * 
	 * @throws IllegalStateException
	 *             Thrown if this instance has a null file reference.
	 */
	public synchronized void validateState() throws IllegalArgumentException,
			IllegalStateException {
		if (file == null) {
			log.error("Unable to start tailer, file not set");
			throw new IllegalStateException(
					"You must set a file to tail before starting the tailer");
		}
	}
}
