/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package nu.zoom.catonine;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Parent class for tailers that notifies listeners of new log blocks.
 * @author Johan Maasing <johan at zoom.nu>
 */
public abstract class AbstractNotifierTailer {

    private final Log log = LogFactory.getLog(getClass());
    private final ArrayList<TailerListener> listeners = new ArrayList<TailerListener>();
    /**
     * The thread group used for notifications and other threads the tailer 
     * might use.
     */
    protected final ThreadGroup tailThreadGroup = new ThreadGroup("Tailer");

    public synchronized void addListener(TailerListener listener) {
        listeners.add(listener);
    }

    public synchronized void removeListener(TailerListener listener) {
        listeners.remove(listener);
    }

    protected synchronized boolean hasListeners() {
        return (listeners.size() > 0);
    }

    /**
     * Informs listeners of new log blocks that have been discovered.
     * @param lines The new log blocks, may not be null.
     */
    protected synchronized void fireLinesRead(List<TailerListener.LogEntry> lines) {
        log.trace("Informing listeners that we have read new lines");
        final List<TailerListener.LogEntry> readonlyList = Collections.unmodifiableList(lines);
        for (final TailerListener listener : listeners) {
            Thread t = new Thread(tailThreadGroup, new Runnable() {

                @Override
                public void run() {
                    listener.lines(readonlyList);
                }
            }, "Lines read notifier");
            t.setDaemon(true);
            log.trace("Starting notifier thread");
            t.start();
        }
    }

    /**
     * Informs listeners that the tailer is working or has become 
     * (temporarily) idle.
     * @param working True if the tailer is performing work, 
     * false to indicate work has stopped and the tailer is idle (for a while).
     */
    protected synchronized void fireWork(final boolean working) {
        log.trace("Informing listeners that we have read new lines");
        for (final TailerListener listener : listeners) {
            Thread t = new Thread(tailThreadGroup, new Runnable() {

                @Override
                public void run() {
                    if (working) {
                        listener.tailerWorking();
                    } else {
                        listener.tailerIdle();
                    }
                }
            }, "Work notifier");
            t.setDaemon(true);
            log.trace("Starting notifier thread");
            t.start();
        }
    }

    /**
     * Inform listeners that the tailer has stopped following (a file) and no
     * more work will be done until the tailer is re-started.
     */
    protected synchronized void fireStopped() {
        log.trace("Informing listeners that we have read new lines");
        for (final TailerListener listener : listeners) {
            Thread t = new Thread(tailThreadGroup, new Runnable() {

                @Override
                public void run() {
                    listener.tailerStopped();
                }
            }, "Work notifier");
            t.setDaemon(true);
            log.trace("Starting notifier thread");
            t.start();
        }
    }
}
