/*
 * Copyright (C) 2008 Johan Maasing johan at zoom.nu Licensed under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package nu.zoom.catonine;

import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Adds methods to a tailer to handle a regular expression that is used to break
 * the logged character stream into 'log blocks'. The characters matching the
 * regular expression is included in the log block. The regular expression
 * can be defined as starting or ending, which means that the matching 
 * characters is etiher included in the start of a log block or at the end.s
 * @author Johan Maasing <johan at zoom.nu>
 */
public abstract class AbstractRegularExpressionLogBlockTailer
        extends AbstractThreadedTailer {

    private final Log log = LogFactory.getLog(getClass());
    private final Pattern DEFAULT_PATTERN = Pattern.compile("((\\n\\r?)|(\\r\\n?))");
    private Pattern pattern = DEFAULT_PATTERN;
    private boolean blockMatcherStarting = false;

    public AbstractRegularExpressionLogBlockTailer(long sleepTime) {
        super(sleepTime);
    }

    /**
     * Set the regular expression that breaks log blocks.
     * @param pattern The pattern, may not be null.
     * @throws java.util.regex.PatternSyntaxException
     */
    public synchronized void setLogBlockRegularExpression(
            Pattern pattern) throws PatternSyntaxException {
        if (log.isTraceEnabled()) {
            log.trace("Changed line ending reg.exp to: " + pattern.toString());
        }
        this.pattern = pattern ;
    }

    /**
     * Get the string representation of the block breaking regular expression.
     * @return
     */
    public synchronized String getLogBlockRegularExpression() {
        return pattern.toString();
    }

    /**
     * Indicates if the characters that breaks a log block is in the start of
     * the log block or in the end of the log block.
     * @return the blockMatcherStarting
     */
    public synchronized boolean isBlockMatcherStarting() {
        return blockMatcherStarting;
    }

    /**
     * Tell if the characters that break a log block is in the start of the log 
     * block or in the end of the block.
     * @param blockMatcherStarting
     *            the blockMatcherStarting to set
     */
    public synchronized void setBlockMatcherStarting(
            boolean blockMatcherStarting) {
        this.blockMatcherStarting = blockMatcherStarting;
    }

    protected Matcher createBlockMatcher() {
        return pattern.matcher("");
    }

    /**
     * Reset the current pattern and block matcher starting to the default value.
     */
    public synchronized void reset() {
        this.pattern = DEFAULT_PATTERN ;
        this.blockMatcherStarting = false ;
    }
}
