/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package nu.zoom.catonine;

import java.io.IOException;
import java.util.ArrayList;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * 
 * @author Johan Maasing <johan at zoom.nu>
 */
public abstract class AbstractThreadedTailer extends AbstractNotifierTailer
        implements Runnable {

    private final Log log = LogFactory.getLog(getClass());
    private Thread tailThread;
    private boolean tail = false;
    private final long MIN_SLEEP_TIME = 200;
    private long sleepTime = MIN_SLEEP_TIME;

    /**
     * Construct a threaded tailer.
     * 
     * @param sleepTime
     *            The time to sleep between runs, clamped to at least 200 ms.
     */
    public AbstractThreadedTailer(final long sleepTime) {
        if (sleepTime > MIN_SLEEP_TIME) {
            this.sleepTime = sleepTime;
        }
        log.trace("Sleep time: " + this.sleepTime);
    }

    @Override
    public synchronized void run() {
        log.trace("Tail thread started");
        while (tail) {
            if (hasListeners()) {
                try {
                    fireWork(true);
                    read();
                    try {
                        fireWork(false);
                        wait(sleepTime);
                    } catch (InterruptedException e) {
                        // We probably was stopped from another thread and the
                        // tail flag should be set to false but set it anyway.
                        tail = false;
                        log.debug("Tail thread interrupted", e);
                        // Set the interrupted flag
                        Thread.currentThread().interrupt();
                    }
                } catch (IOException e) {
                    ArrayList<TailerListener.LogEntry> errorMessages = new ArrayList<TailerListener.LogEntry>();
                    errorMessages.add(new TailerListener.LogEntry(0, "IOException while tailing"));
                    errorMessages.add(new TailerListener.LogEntry(26, e.toString()));
                    errorMessages.add(new TailerListener.LogEntry(e.toString().length() + 26, "Aborting tail thread"));
                    fireLinesRead(errorMessages);
                    tail = false;
                    log.error("Aborting tail thread", e);
                }
            }
        }
        log.trace("Tail thread exiting");
        stopped();
        fireStopped();
    }

    /**
     * Indicates if the tailer is tailing.
     * 
     * @return true if the tailer has not been told to stop and is active.
     */
    public synchronized boolean isTailing() {
        return ((tail) && (tailThread.isAlive()));
    }

    /**
     * Tell the tailer to start tailing. Subclasses should implement the setup 
     * method to initialize the state and the validateState method to signal 
     * if it can not be started.
     * 
     * @throws java.lang.IllegalStateException
     *             If the tailer is not in a state to be started.
     */
    public synchronized void start() throws IllegalStateException {
        setup();
        validateState();
        if (tailThread == null) {
            log.trace("Initializing tail thread");
            tail = true;
            tailThread = new Thread(tailThreadGroup, this, "Tail thread");
            tailThread.setDaemon(true);
            tailThread.start();
        }
    }

    protected abstract void setup();

    /**
     * Subclasses can override this method to perform state validation before
     * the tailer starts calling the read method.
     * 
     * @throws IllegalStateException
     *             If the tailer is not in a state to be started.
     */
    protected abstract void validateState() throws IllegalStateException;

    /**
     * Inform the tailer to stop. Subclasses should override the
     * 
     * @link{#stopped} method to perform clean up.
     */
    public synchronized void stop() {
        log.trace("Stopping tail thread");
        tail = false;
        if (tailThread != null) {
            log.trace("Interrupting tail thread");
            tailThread.interrupt();
            tailThread = null;
        }
    }

    /**
     * Subclasses can implement this method to perform resource clean up after
     * the tailer has stopped calling the read method.
     */
    protected void stopped() {
    }

    /**
     * Subclasses should override this method and try to detect new log blocks.
     * If new blocks are detected the class should fire an event to registered
     * listeners to let them know of the new log blocks.
     * 
     * @throws java.io.IOException
     *             May be used to signal that reading failed. This will cause
     *             the tailer to stop.
     */
    protected abstract void read() throws IOException;
}
