/*
 * Copyright (C) 2008 Johan Maasing johan at zoom.nu Licensed under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package nu.zoom.catonine;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.channels.FileChannel;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public final class SlidingWindowTailer extends AbstractFileTailer {

    private final Log log = LogFactory.getLog(getClass());
    private FileChannel fileChannel = null;
    private ByteBuffer readBuffer = null;
    private final static int DEFAULT_BUFFER_SIZE = 1024 * 32;

    public SlidingWindowTailer() {
        this(null, null, 500);
    }

    public SlidingWindowTailer(File file, Charset charset, long sleepTime) {
        super(file, charset, sleepTime);
        log.trace("Creating tailer");

        setReadBufferSize(DEFAULT_BUFFER_SIZE);
    }

    public synchronized void setReadBufferSize(int size) {
        // Lets make sure that at least an EOL fits in the buffer, not that it
        // matters but really small buffer sizes makes no sense.
        if (size < 3) {
            throw new IllegalArgumentException(
                    "Buffer size may not be less than 3");
        }
        log.debug("Changing Readbuffer size to: " + size);
        readBuffer = ByteBuffer.allocate(size);
        log.trace("Readbuffer allocated");
    }

    @Override
    public synchronized void reset() {
        super.reset();
        setReadBufferSize(DEFAULT_BUFFER_SIZE);
    }

    @Override
    public synchronized void validateState() throws IllegalArgumentException,
            IllegalStateException {
        super.validateState();
        if ((fileChannel == null) || (!fileChannel.isOpen())) {
            log.error("Filechannel null or not opened");
            throw new IllegalStateException("File channel is not set or not open");
        }
    }

    @Override
    protected void setup() {
        FileInputStream fis;
        try {
            fis = new FileInputStream(getFile());
            fileChannel = fis.getChannel();
        } catch (IOException ex) {
            log.error(ex);
            if (fileChannel != null) {
                try {
                    fileChannel.close();
                } catch (IOException ex1) {
                    log.error(ex1);
                }
            }
            fileChannel = null;
        }
    }

    @Override
    protected synchronized void stopped() {
        if (fileChannel != null) {
            try {
                log.trace("Closing file channel");
                fileChannel.close();
            } catch (IOException ex) {
                log.error("Unable to close file channel", ex);
            }
        }
    }

    @Override
    protected synchronized void read() throws IOException {
        log.trace("Reading file");
        final long fileOffset = calculateStartPosition();
        if (fillBuffer(fileOffset) > -1) {
            log.trace("Flipping read buffer");
            readBuffer.flip();
            log.trace("Converting buffer to lines.");
            List<TailerListener.LogEntry> lines = convertToLines(readBuffer, fileOffset);
            log.trace("Informing listeners that lines have been read");
            fireLinesRead(lines);
        }
    }

    private synchronized long calculateStartPosition() throws IOException {
        long fileSize = fileChannel.size();
        long startPosition = fileSize - readBuffer.capacity();
        if (startPosition < 0) {
            log.debug("File read start position clamped to 0");
            startPosition = 0;
        }
        log.debug("File read start position: " + startPosition);
        return startPosition;
    }

    private synchronized int fillBuffer(final long fileOffset) throws IOException {
        fileChannel.position(fileOffset);
        if (fileChannel != null) {
            log.debug("Clearing read buffer");
            readBuffer.clear();
            return fileChannel.read(readBuffer);
        } else {
            log.warn("No file channel set, will not fill read buffer");
            return -1;
        }
    }

    private synchronized List<TailerListener.LogEntry> convertToLines(final ByteBuffer buffer, final long fileoffset) {
        Charset charset = getCharset();
        log.trace("Decoding read buffer to characters using charset: " + charset);
        CharBuffer chBuffer = charset.decode(buffer);
        ArrayList<TailerListener.LogEntry> lines = new ArrayList<TailerListener.LogEntry>();

        Matcher blockMatcher = createBlockMatcher();
        blockMatcher.reset(chBuffer);

        log.trace("Breaking buffer into blocks using pattern: " + blockMatcher.pattern().toString());
        if (blockMatcher.find()) {
            int startOfBlock = 0;
            int endOfBlock = 0;
            CharSequence blockSeq;
            // Deal with the first block
            endOfBlock = (isBlockMatcherStarting()) ? blockMatcher.start()
                    : blockMatcher.end();
            blockSeq = chBuffer.subSequence(startOfBlock, endOfBlock);
            lines.add(new TailerListener.LogEntry(fileoffset + startOfBlock, blockSeq.toString()));
            startOfBlock = endOfBlock;
            // Deal with middle
            while (blockMatcher.find()) {
                endOfBlock = (isBlockMatcherStarting()) ? blockMatcher.start()
                        : blockMatcher.end();
                blockSeq = chBuffer.subSequence(startOfBlock, endOfBlock);
                lines.add(new TailerListener.LogEntry(fileoffset + startOfBlock, blockSeq.toString()));
                startOfBlock = endOfBlock;
            }
            // Deal with the remainder
            blockSeq = chBuffer.subSequence(endOfBlock, chBuffer.remaining());
            lines.add(new TailerListener.LogEntry(fileoffset + endOfBlock, blockSeq.toString()));
        } else {
            if (log.isTraceEnabled()) {
                log.trace("No match in buffer for block pattern: " + blockMatcher.pattern() + " using entire buffer as block");
            }
            // No endings at all, use entire buffer as a block
            CharSequence lineSeq = chBuffer.subSequence(0, chBuffer.remaining());
            String line = lineSeq.toString();
            lines.add(new TailerListener.LogEntry(fileoffset + 0, line));
        }
        if (log.isTraceEnabled()) {
            log.debug("Found " + lines.size() + " lines");
        }
        return lines;
    }

    @Override
    protected void fileToTailChanged(File file) throws FileNotFoundException {
        FileInputStream fis = new FileInputStream(file);
        fileChannel = fis.getChannel();
    }

    @Override
    protected void charsetChanged(Charset newCharset) {
        // Nothing we care about in this call back, charset is read when needed
    }
}
