/*
 * Copyright (C) 2006 Johan Maasing johan at zoom.nu Licensed under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package nu.zoom.catonine.desktop;

import java.awt.EventQueue;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.io.File;
import java.util.Set;
import javax.swing.JComponent;

import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;
import nu.zoom.catonine.ui.LocalizedResources;
import nu.zoom.catonine.ui.LocalizedResources.ResourceNotFoundException;

import nu.zoom.swing.desktop.PlugIn;
import nu.zoom.swing.desktop.Workbench;
import nu.zoom.swing.desktop.WorkbenchListener;
import nu.zoom.swing.desktop.WorkbenchMenuBar;
import nu.zoom.swing.desktop.component.filechooser.FileChooserFactory;
import nu.zoom.swing.desktop.component.stringmenu.StringMenu;
import nu.zoom.swing.desktop.component.stringmenu.StringMenuFactory;
import nu.zoom.swing.desktop.preferences.Preferences;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.ops4j.gaderian.Messages;

public class DesktopPlugIn implements PlugIn, WorkbenchListener {

    private final Log log = LogFactory.getLog(getClass());
    private final Workbench workbench;
    private final Messages messages;
    private final FileChooserFactory chooserFactory;
    private final StyleRulesPlugIn styleRulesPlugIn;
    private final StringMenuFactory menuFactory;
    private final Preferences preferences;
    private final FileReopener reOpener;
    private FileOpener fileOpener;
    static final String FILE_OPEN_PREFERENCESKEY = "svansprogram-file";
    static final String FILE_RECENT_PREFERENCESKEY = "svansprogram-recent";

    public DesktopPlugIn(
            final Workbench workbench,
            final Messages messages,
            final StringMenuFactory menuFactory,
            final FileChooserFactory chooserFactory,
            final StyleRulesPlugIn styleRulesPlugIn,
            final Preferences preferences,
            final FileReopener reOpener) {
        super();
        log.trace("Instantiating plugin");
        this.workbench = workbench;
        this.messages = messages;
        this.chooserFactory = chooserFactory;
        this.styleRulesPlugIn = styleRulesPlugIn;
        this.menuFactory = menuFactory;
        this.preferences = preferences;
        this.reOpener = reOpener;
    }

    @Override
    public void initialize() throws Exception {
        log.trace("Registering as a workbench listener");
        workbench.addWorkBenchListener(this);
    }

    @Override
    public void close() {
    }

    @Override
    public void start() {
        log.trace("Registering plugin on the workbench menu");

        LocalizedResources localizedResources = new LocalizedResources(messages);
        
        fileOpener = new FileOpener(workbench, localizedResources, preferences, styleRulesPlugIn, messages, reOpener);
        OpenFileAction openAction;
        try {
            openAction = new OpenFileAction(
                    workbench,
                    localizedResources,
                    chooserFactory,
                    menuFactory,
                    styleRulesPlugIn,
                    preferences,
                    messages,
                    fileOpener);

            WorkbenchMenuBar menuBar = workbench.getMenuBar();
            menuBar.addToApplicationMenu(new JMenuItem(openAction));
            String recentFilesName = messages.getMessage("nu.zoom.catonine.recent");
            StringMenu<File> recentMenu = menuFactory.getMenu(File.class, FILE_RECENT_PREFERENCESKEY,
                    recentFilesName, null);
            recentMenu.addListener(openAction);
            JMenu recentJMenu = recentMenu.getJMenu();
            recentJMenu.setMnemonic(KeyEvent.VK_R);
            menuBar.addToApplicationMenu(recentJMenu);

            workbench.registerKeyboardAction(
                    openAction,
                    KeyStroke.getKeyStroke(KeyEvent.VK_O, InputEvent.CTRL_DOWN_MASK),
                    JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);

            EventQueue.invokeLater(new Runnable() {

                @Override
                public void run() {
                    reOpenFiles();
                }
            });
        } catch (ResourceNotFoundException ex) {
            log.error(ex);
            workbench.getErrorReporter().reportError(ex);
        }
    }

    private void reOpenFiles() {
        Set<String> rememeberedFiles = reOpener.getRememeberedFiles();
        for (String filename : rememeberedFiles) {
            log.debug("Reopening file: " + filename);
            File file = new File(filename);
            if (file.canRead()) {
                try {
                    fileOpener.openFile(file);
                } catch (ResourceNotFoundException ex) {
                    log.info("Missing configuration", ex);
                }
            } else {
                log.info("Unable to reopen file for reading: " + filename);
            }
        }

    }
}
