/*
 *  Copyright 2010 johan.
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 * 
 *       http://www.apache.org/licenses/LICENSE-2.0
 * 
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *  under the License.
 */
package nu.zoom.catonine.desktop;

import java.awt.Component;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import nu.zoom.catonine.swing.TailPane;
import nu.zoom.catonine.ui.LocalizedResources;
import nu.zoom.catonine.ui.LocalizedResources.ResourceNotFoundException;
import nu.zoom.swing.desktop.Workbench;
import nu.zoom.swing.desktop.WorkbenchFrame;
import nu.zoom.swing.desktop.WorkbenchFrameListener;
import nu.zoom.swing.desktop.common.BackendException;
import nu.zoom.swing.desktop.common.action.DetachAction;
import nu.zoom.swing.desktop.preferences.InvalidDataTypeException;
import nu.zoom.swing.desktop.preferences.Preferences;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.ops4j.gaderian.Messages;

/**
 *
 * @author Johan Maasing
 */
public final class FileOpener {

    private final Log log = LogFactory.getLog(getClass());
    private final Workbench workbench;
    private final LocalizedResources localizedResources;
    private final Preferences preferences;
    private final StyleRulesPlugIn styleRulesPlugIn;
    private final Messages messages;
    private final FileReopener reOpener;

    public FileOpener(
            final Workbench workbench,
            final LocalizedResources localizedResources,
            final Preferences preferences,
            final StyleRulesPlugIn styleRulesPlugIn,
            final Messages messages,
            final FileReopener reOpener) {
        this.workbench = workbench;
        this.localizedResources = localizedResources;
        this.preferences = preferences;
        this.styleRulesPlugIn = styleRulesPlugIn;
        this.messages = messages;
        this.reOpener = reOpener;
    }

    void openFile(final File file) throws ResourceNotFoundException {
        log.trace("Opening selected file");

        ArrayList<Component> additionalToolbarComponents =
                new ArrayList<Component>();
        JButton detachToggle = new JButton();
        additionalToolbarComponents.add(detachToggle);
        TailPane pane = new TailPane(
                localizedResources,
                additionalToolbarComponents,
                preferences);
        try {
            log.trace("Setting style rules manager");
            pane.setStyleRulesManager(styleRulesPlugIn.getStyleRulesManager());
            log.trace("Setting file to tail");
            pane.setFile(file);
            log.trace("Creating workbench frame");
            final String prefskey = "Svansprogram.file." + file.getName();
            WorkbenchFrame frame = workbench.createWorkbenchFrame(
                    prefskey, pane, true, true);
            frame.addFrameListener(new TailerStopper(pane));
            frame.addFrameListener(new TailFrameTracker(reOpener, file));
            frame.setTitle(createTitle(file));

            List<ImageIcon> imageIcons = Svansprogram.getTailIcon();
            frame.setFrameIcon(imageIcons);

            detachToggle.setAction(new DetachAction(messages, frame));

            log.trace("Showing workbench frame");
            frame.setVisible(true);
            final String configurationUUID = getPreferredConfigurationUUID(file);
            if (configurationUUID != null) {
                pane.setPreferredConfiguration(configurationUUID);
            }
            workbench.setStatusbarMessage(localizedResources.format(
                    "nu.zoom.catonine.opened", file.getAbsolutePath()));
        } catch (Exception e1) {
            log.error("Unable to create tail pane", e1);
            String message = localizedResources.getMessage("nu.zoom.catonine.open.fail");
            workbench.getErrorReporter().reportError(message, e1);
        }
    }

    private String createTitle(File file) {
        File absoluteFile = file.getAbsoluteFile();
        String title = absoluteFile.getName() + "@" + absoluteFile.getParentFile().toString();
        return title;
    }

    private String getPreferredConfigurationUUID(final File file) throws InvalidDataTypeException, BackendException {
        if (file == null) {
            throw new NullPointerException("File may not be null");
        }
        final String uuid = PreferencesUtil.getPreferredUUIDForFile(preferences, file);
        log.debug("File: " + file.getName() + " is associated with configuration UUID: " + uuid);
        return uuid;

    }

    class TailerStopper implements WorkbenchFrameListener {

        TailPane pane;

        public TailerStopper(TailPane pane) {
            super();
            this.pane = pane;
        }

        @Override
        public void frameClosed(WorkbenchFrame frame) {
            // Nothing to do here
        }

        @Override
        public void frameWillDispose(WorkbenchFrame frame) {
            try {
                log.trace("Frame will dispose, stopping tail pane");
                pane.stop();
            } catch (IOException e) {
                log.error("Error stopping tail pane", e);
            }
        }
    }
}
