/*
 * Copyright (C) 2006 Johan Maasing johan at zoom.nu Licensed under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package nu.zoom.catonine.desktop;

import java.awt.event.InputEvent;
import java.io.IOException;
import nu.zoom.catonine.SampleStyleRulesFactory;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.ObjectInputStream;
import java.util.ArrayList;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComponent;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.KeyStroke;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;

import nu.zoom.catonine.StyleRulesManager;
import nu.zoom.catonine.StyleRulesManagerListener;
import nu.zoom.catonine.swing.ImportStyleRulesManagerAction;
import nu.zoom.catonine.swing.OpenStyleRulesManagerAction;
import nu.zoom.catonine.swing.SaveStyleRulesManagerAsAction;
import nu.zoom.catonine.ui.LocalizedResources;
import nu.zoom.catonine.ui.LocalizedResources.ResourceNotFoundException;
import nu.zoom.swing.desktop.CloseVetoListener;
import nu.zoom.swing.desktop.Workbench;
import nu.zoom.swing.desktop.WorkbenchFrame;
import nu.zoom.swing.desktop.WorkbenchListener;
import nu.zoom.swing.desktop.WorkbenchMenuBar;
import nu.zoom.swing.desktop.common.BackendException;
import nu.zoom.swing.desktop.common.action.ToggleFrameAction;
import nu.zoom.swing.desktop.component.filechooser.FileChooser;
import nu.zoom.swing.desktop.component.filechooser.FileChooserFactory;
import nu.zoom.swing.desktop.preferences.InvalidDataTypeException;
import nu.zoom.swing.desktop.preferences.Preferences;

import org.ops4j.gaderian.Messages;

/**
 * 
 * @author $Author: $
 * @version $Revision: $
 * 
 */
public class StyleRulesPlugInImpl implements StyleRulesPlugIn,
        WorkbenchListener, CloseVetoListener {

    private StyleRulesManager manager;
    private final Workbench workbench;
    private final Messages messages;
    private final LocalizedResources localizedResources;
    private final Preferences preferences;
    private FileChooserFactory fileChooserFactory;
    final ArrayList<StyleRulesManagerListener> listeners = new ArrayList<StyleRulesManagerListener>();
    StyleRulesManagerComponent styleRulesManagerComponent;

    public StyleRulesPlugInImpl(Workbench workbench, Messages messages,
            Preferences preferences, FileChooserFactory fileChooserFactory) {
        super();
        this.workbench = workbench;
        this.messages = messages;
        this.localizedResources = new LocalizedResources(messages);
        this.preferences = preferences;
        this.fileChooserFactory = fileChooserFactory;
    }

    /*
     * (non-Javadoc)
     *
     * @see nu.zoom.swing.desktop.PlugIn#initialize()
     */
    @Override
    public void initialize() throws Exception {
        workbench.addWorkBenchListener(this);
    }

    /*
     * (non-Javadoc)
     *
     * @see nu.zoom.swing.desktop.WorkbenchListener#start()
     */
    @Override
    public void start() {
        final ShowStyleRulesAction showStyleRulesAction = new ShowStyleRulesAction(messages,
                "nu.zoom.catonine.configurations");
        final WorkbenchMenuBar menuBar = workbench.getMenuBar();
        menuBar.addToApplicationMenu(showStyleRulesAction.getControl());
        try {
            menuBar.addToApplicationMenu(new JMenuItem(new SaveStyleRulesManagerAsAction(this, localizedResources)));
            menuBar.addToApplicationMenu(new JMenuItem(new OpenStyleRulesManagerAction(this, localizedResources)));
            menuBar.addToApplicationMenu(new JMenuItem(new ImportStyleRulesManagerAction(this, localizedResources)));
        } catch (LocalizedResources.ResourceNotFoundException resourceNotFoundException) {
            workbench.getErrorReporter().reportError(resourceNotFoundException);
        }
        workbench.addCloseVetoListener(this);
        workbench.registerKeyboardAction(
                showStyleRulesAction,
                KeyStroke.getKeyStroke(KeyEvent.VK_N, InputEvent.CTRL_DOWN_MASK),
                JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
        try {
            if (!loadManager()) {
                manager = new StyleRulesManager();
                manager.add(SampleStyleRulesFactory.getLog4JStyleRules());
                manager.add(SampleStyleRulesFactory.getSyslogStyleRules());
            }
        } catch (InvalidDataTypeException invalidDataTypeException) {
            workbench.getErrorReporter().reportError(invalidDataTypeException);
        } catch (BackendException backendException) {
            workbench.getErrorReporter().reportError(backendException);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see nu.zoom.swing.desktop.WorkbenchListener#close()
     */
    @Override
    public void close() {
    }

    /*
     * (non-Javadoc)
     *
     * @see nu.zoom.swing.desktop.CloseVetoListener#canClose()
     */
    @Override
    public boolean canClose() {
        if (manager.isChanged()) {
            int result = JOptionPane.showConfirmDialog(
                    workbench.getDialogOwner(),
                    messages.getMessage("nu.zoom.catonine.unsaved"),
                    messages.getMessage("nu.zoom.catonine.unsaved.title"),
                    JOptionPane.YES_NO_CANCEL_OPTION);
            switch (result) {
                case JOptionPane.YES_OPTION:
                    saveManager(false);
                    return true;
                case JOptionPane.NO_OPTION:
                    return true;
                default:
                    return false;
            }
        }
        return true;
    }

    public class ShowStyleRulesAction extends ToggleFrameAction<JCheckBoxMenuItem> {

        private static final long serialVersionUID = 1L;

        public ShowStyleRulesAction(Messages messages, String nameKey) {
            super(messages, new JCheckBoxMenuItem());
            setNameFromMessages(nameKey);
            setMnemonicKey(KeyEvent.VK_N);
            setAcceleratorKey(KeyEvent.VK_N, KeyEvent.CTRL_DOWN_MASK);
        }

        /*
         * (non-Javadoc)
         *
         * @see nu.zoom.swing.desktop.component.ToggleFrameAction#createFrame()
         */
        @Override
        protected WorkbenchFrame createFrame() {
            try {
                final StyleRulesManagerComponent newStyleRulesManagerComponent =
                        new StyleRulesManagerComponent(manager, localizedResources, workbench);
                if (StyleRulesPlugInImpl.this.styleRulesManagerComponent != null) {
                    listeners.remove(StyleRulesPlugInImpl.this.styleRulesManagerComponent);
                }
                listeners.add(newStyleRulesManagerComponent);
                StyleRulesPlugInImpl.this.styleRulesManagerComponent = newStyleRulesManagerComponent;
                String title = null;
                try {
                    title = preferences.getString(PreferencesUtil.MANAGER_FILENAME_PREFS_KEY);
                    if (title != null) {
                        File titleFile = new File(title);
                        title = titleFile.getName();
                    }
                } catch (InvalidDataTypeException ex) {
                    // TODO: Log error. But ignore since we will use default for title
                } catch (BackendException ex) {
                    // TODO: Log error. But ignore since we will use default for title
                }
                if (title == null) {
                    title = messages.getMessage("nu.zoom.catonine.configurations");
                }
                WorkbenchFrame managerFrame = workbench.createWorkbenchFrame(
                        "StyleRulesColoring", newStyleRulesManagerComponent, true, true);
                managerFrame.setTitle(title);
                return managerFrame;
            } catch (LocalizedResources.ResourceNotFoundException resourceNotFoundException) {
                // TODO: Log error
                return null;
            }
        }
    }

    private boolean loadManager() throws InvalidDataTypeException, BackendException {
        boolean success = loadManager(preferences.getString(PreferencesUtil.MANAGER_FILENAME_PREFS_KEY));
        if (!success) {
            // Failed to load default manager. Remove the preferences string since it is obviously wrong.
            preferences.setString(PreferencesUtil.MANAGER_FILENAME_PREFS_KEY, null);
        }
        return success;
    }

    public void importManager() {
        if (this.manager == null) {
            openManager();
        } else {
            final FileChooser fileChooser = fileChooserFactory.getFileChooser(PreferencesUtil.MANAGER_FILECHOOSER_PREFS_KEY);
            File openFile = fileChooser.openFile();
            if (openFile != null) {
                StyleRulesManager importFrom = readManager(openFile);
                if (importFrom != null) {
                    workbench.startWorkIndicator();
                    this.manager.importManager(importFrom);
                    workbench.stopWorkIndicator();
                }
            }
        }
    }

    public boolean openManager() {
        final FileChooser fileChooser = fileChooserFactory.getFileChooser(PreferencesUtil.MANAGER_FILECHOOSER_PREFS_KEY);
        File openFile = fileChooser.openFile();
        if (openFile != null) {
            try {
                final boolean success = loadManager(openFile.getCanonicalPath());
                if (success) {
                    fireStyleRulesManagerChanged();
                }
                return success;
            } catch (IOException ex) {
                try {
                    String message = localizedResources.getMessage("nu.zoom.catonine.configurations.load.failed");
                    workbench.getErrorReporter().reportError(message, ex);
                } catch (ResourceNotFoundException ex1) {
                    workbench.getErrorReporter().reportError("Unable to load resource", ex);
                }
            }
        }
        return false;
    }

    private boolean loadManager(final String managerFileName) {
        if (managerFileName == null) {
            return false;
        }
        if ((manager != null) && (manager.isChanged())) {
            saveManager(false);
        }
        StyleRulesManager newManager = readManager(managerFileName);
        // If we load a new manager when an old is opened we should inform
        // listeners (GUI elements) that the rules have changed.
        if (newManager != null) {
            if (manager != null) {
                manager.copyListenersTo(newManager);
            }
            manager = newManager;
            try {
                preferences.setString(PreferencesUtil.MANAGER_FILENAME_PREFS_KEY, managerFileName);
            } catch (BackendException ex) {
                // TODO: log error but we ignore it. If not set it only affects
                // the frame title.
            }
            return true;
        }
        return false;
    }

    private StyleRulesManager readManager(String managerFileName) {
        StyleRulesManager newManager = null;
        if (managerFileName != null) {
            final File managerFile = new File(managerFileName);
            newManager = readManager(managerFile);
        }
        return newManager;
    }

    private StyleRulesManager readManager(File managerFile) {
        StyleRulesManager newManager = null;
        try {
            if (!managerFile.exists()) {
                final String message = messages.format("nu.zoom.catonine.open.fail.noexist", managerFile.getCanonicalPath());
                workbench.getErrorReporter().reportError(message);
            } else if (!managerFile.canRead()) {
                final String message = messages.format("nu.zoom.catonine.open.fail.noread", managerFile.getCanonicalPath());
                workbench.getErrorReporter().reportError(message);
            } else {
                if (managerFile.getName().endsWith("xml")) {
                    // Use JAXB-format
                    JAXBContext jc = JAXBContext.newInstance(StyleRulesManager.class);
                    Unmarshaller unmarshaller = jc.createUnmarshaller();
                    newManager = (StyleRulesManager) unmarshaller.unmarshal(managerFile);
                } else {
                    // Old school serialized manager
                    ObjectInputStream objIns = new ObjectInputStream(
                            new FileInputStream(managerFile));
                    newManager = (StyleRulesManager) objIns.readObject();
                }
            }
        } catch (Exception e) {
            workbench.getErrorReporter().reportError(
                    messages.getMessage("nu.zoom.catonine.configurations.load.failed"),
                    e);
            newManager = null;
        }
        return newManager;
    }

    public void saveManager(final boolean saveAs) {
        try {
            String managerFileName = preferences.getString(PreferencesUtil.MANAGER_FILENAME_PREFS_KEY);
            // there is no default name or we should select a new name
            if ((managerFileName == null) || saveAs) {
                FileChooser fileChooser = fileChooserFactory.getFileChooser(PreferencesUtil.MANAGER_FILECHOOSER_PREFS_KEY);
                File selectedSaveFile = fileChooser.saveFile();
                if (selectedSaveFile != null) {
                    managerFileName = selectedSaveFile.getCanonicalPath();
                }
            }
            // make sure it ends with .xml
            if (managerFileName != null) {
                if (!managerFileName.endsWith(".xml")) {
                    managerFileName = managerFileName + ".xml";
                }
                final File saveFile = new File(managerFileName);
                JAXBContext jc = JAXBContext.newInstance(StyleRulesManager.class);
                Marshaller marshaller = jc.createMarshaller();
                marshaller.setProperty("jaxb.formatted.output", true);
                marshaller.marshal(manager, saveFile);
                manager.resetChangeStatus();
                preferences.setString(PreferencesUtil.MANAGER_FILENAME_PREFS_KEY, saveFile.getCanonicalPath());
            }
        } catch (Exception e) {
            workbench.getErrorReporter().reportError(
                    messages.getMessage("nu.zoom.catonine.configurations.save.failed"),
                    e);
        }
    }

    private void fireStyleRulesManagerChanged() {
        for (StyleRulesManagerListener styleRulesManagerListener : listeners) {
            styleRulesManagerListener.styleRulesChanged(manager);
        }
    }

    @Override
    public StyleRulesManager getStyleRulesManager() {
        return manager;
    }
}
