/*
 * Copyright (C) 2006,2008 Johan Maasing johan at zoom.nu Licensed under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package nu.zoom.catonine.swing;

import java.awt.EventQueue;
import java.awt.Font;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.regex.Matcher;
import javax.swing.text.AttributeSet;

import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultStyledDocument;
import javax.swing.text.Position;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;

import nu.zoom.catonine.StyleRule;
import nu.zoom.catonine.StyleRules;
import nu.zoom.catonine.TailerListener;

/**
 * Styled document that listens for new lines to add and colors the lines
 * according to the selected style rules.
 * @author Johan Maasing
 */
public class StyleRulesDocument extends DefaultStyledDocument implements
        TailerListener {

    private static final long serialVersionUID = 1L;
    final static String DEFAULT_STYLE_NAME = "default";
    private StyleRules configuration = null;
    protected List<LogEntry> fileContent = null;
    protected final Position startPosition;
    protected final Position endPosition;

    public StyleRulesDocument() {
        super();
        startPosition = getStartPosition();
        endPosition = getEndPosition();
    }

    protected void setConfiguration(StyleRules configuration) {
        this.configuration = configuration;
        restyle();
    }

    protected synchronized void restyle() {
        EventQueue.invokeLater(new Runnable() {

            @Override
            public void run() {
                try {
                    writeLock();
                    setupStyles();

                    // Clear the document
                    int start = startPosition.getOffset();
                    int length = endPosition.getOffset() - 1;
                    try {
                        remove(start, length);
                    } catch (BadLocationException e) {
                        // if this happens there is something seriously wrong
                        // with the lock on the document
                        throw new IllegalStateException(e);
                    }

                    // Add the buffer
                    if (fileContent != null) {
                        append(fileContent);
                    }
                } finally {
                    writeUnlock();
                }
            }
        });
    }

    /**
     * Get the styled document style name that should be used for the given
     * log block. Matches the given string against the style rules to determine
     * which style to use.
     * @param logblock The string that should be styled.
     * @return The style. Null if the block should be supressed, i.e. not drawn
     * at all.
     */
    protected String getStyleName(String logblock) {
        StyleRule match = null;
        if (configuration != null) {
            Iterator<StyleRule> patternIterator =
                    configuration.getPatternStyles().iterator();
            while ((match == null) && patternIterator.hasNext()) {
                StyleRule config = patternIterator.next();
                if (!config.isDisabled()) {
                    Matcher matcher = config.getPattern().matcher(logblock);
                    if (matcher.matches()) {
                        match = config;
                    }
                }
            }
        }
        if (match != null) {
            if (!match.isSupress()) {
                return match.getGuid();
            } else {
                return null;
            }
        } else {
            return DEFAULT_STYLE_NAME;
        }
    }

    /**
     * Append a list of log blocks to the document.
     * @param blocks The list of log blocks to append to the document.
     */
    protected void append(final List<LogEntry> blocks) {
        for (LogEntry block : blocks) {
            String style = getStyleName(block.getContent());
            if (style != null) {
                append(block.getContent(), style);
            } else {
                // Style == null means that the line should be
                // supressed
            }
        }
    }

    /**
     * Append a string to the styled document with the given style name.
     * @param message The message to append, if null this method does nothing.
     * @param styleName The style name to apply, should not be null.
     */
    protected void append(final String message, final String styleName) {
        final Style style = getStyle(styleName);
        append(message, style);
    }

    protected void append(final String message, final AttributeSet style) {
        // Despite the documentation the insertString behaves differently if not
        // invoked on the event queue. The Caret does not move which means that
        // it will not scroll to show the inserted text.
        // http://groups.google.com/group/comp.lang.java.gui/browse_thread/thread/24aeefd624056bdc/acd3bb7fb06652a1?#acd3bb7fb06652a1

        try {
            final int offset = this.endPosition.getOffset() - 1;
            insertString(offset, message, style);
        } catch (BadLocationException exc) {
            // if this happens there is something seriously wrong with the
            // lock on the document
            throw new IllegalStateException(exc);
        }
    }

    protected synchronized void setupStyles() {
        Enumeration<?> styleNameEnumeration = getStyleNames();
        while (styleNameEnumeration.hasMoreElements()) {
            removeStyle((String) styleNameEnumeration.nextElement());
        }
        Style defaultStyle = addStyle(DEFAULT_STYLE_NAME, null);

        StyleConstants.setFontFamily(defaultStyle, Font.MONOSPACED);
        if (configuration != null) {
            List<StyleRule> configurations = configuration.getPatternStyles();
            if (configurations != null) {
                for (StyleRule styleRule : configurations) {
                    Style configStyle = addStyle(styleRule.getGuid(),
                            defaultStyle);
                    if (styleRule.getBackground() != null) {
                        StyleConstants.setBackground(
                                configStyle,
                                styleRule.getBackground());
                    }
                    if (styleRule.getForeground() != null) {
                        StyleConstants.setForeground(
                                configStyle,
                                styleRule.getForeground());
                    }
                }
            }
        }
    }

    @Override
    public synchronized void lines(List<LogEntry> lines) {
        // This is for use with the windowed tailer that only looks at a
        // block of the file. This document will be reset for each call and
        // restyled entirely
        fileContent = lines;
        restyle();
    }

    @Override
    public void tailerIdle() {
    }

    @Override
    public void tailerWorking() {
    }

    @Override
    public void tailerStopped() {
    }
}
