/*
 * Copyright (C) 2006 Johan Maasing johan at zoom.nu Licensed under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package nu.zoom.catonine.swing;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.EventQueue;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.List;
import javax.swing.Box;

import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.SwingConstants;

import nu.zoom.catonine.StyleRule;
import nu.zoom.catonine.StyleRuleListener;
import nu.zoom.catonine.StyleRules;
import nu.zoom.catonine.StyleRulesListModel;
import nu.zoom.catonine.StyleRulesListener;
import nu.zoom.catonine.StyleRulesManager;
import nu.zoom.catonine.SlidingWindowTailer;
import nu.zoom.catonine.TailerListener;
import nu.zoom.catonine.desktop.PreferencesUtil;
import nu.zoom.catonine.ui.LocalizedResources;
import nu.zoom.catonine.ui.LocalizedResources.ResourceNotFoundException;
import nu.zoom.swing.desktop.common.BackendException;
import nu.zoom.swing.desktop.preferences.InvalidDataTypeException;
import nu.zoom.swing.desktop.preferences.Preferences;

/**
 * The main panel of the tailing component.
 */
public class TailPane extends JPanel implements StyleRulesListener,
        StyleRuleListener, TailerListener {

    private static final long serialVersionUID = 1L;
    private StyleRuleModel listModel;
    private StyleRulesManager manager = null;
    private final BorderLayout mainLayout = new BorderLayout();
    private final JScrollPane contentScrollPane;
    private final TimestampDocument document;
    private final JTextPane textPane;
    private final JButton tailControl;
    private final JComboBox configurationList;
    private StyleRules selectedConfiguration = null;
    private final SlidingWindowTailer tailer;
    private final JLabel tailerMessageLabel;
    private final Icon tailerBusyIcon;
    private final Icon tailerWaitingIcon;
    private final Icon tailerStoppedIcon;
    private final Preferences preferences;
    private final JPopupMenu tailMenu;

    /**
     * Create the tail panel.
     * @param The resource bundle to use.
     * @param additionalToolbarComponents
     *            Additional components that are to be placed (first) on the
     *            toolbar.
     */
    public TailPane(
            final LocalizedResources localizedResources,
            final List<Component> additionalToolbarComponents,
            final Preferences preferences) throws ResourceNotFoundException {
        super();
        this.preferences = preferences;

        tailerBusyIcon = localizedResources.getIcon("nu.zoom.catonine.tail.icon.busy");
        tailerWaitingIcon = localizedResources.getIcon("nu.zoom.catonine.tail.icon.idle");
        tailerStoppedIcon = localizedResources.getIcon("nu.zoom.catonine.tail.icon.stopped");

        document = new TimestampDocument();
        tailer = new SlidingWindowTailer();

        setLayout(mainLayout);

        // Build tail -f pane
        textPane = new JTextPane(document);
        textPane.setEditable(false);
        contentScrollPane = new JScrollPane(textPane);
        add(contentScrollPane, BorderLayout.CENTER);

        JPanel noWrapPanel = new JPanel(new BorderLayout());
        noWrapPanel.add(textPane);
        setLayout(new BorderLayout());
        JScrollPane textScroller = new JScrollPane(noWrapPanel);
        textScroller.getVerticalScrollBar().setUnitIncrement(16);
        textScroller.getHorizontalScrollBar().setUnitIncrement(16);
        add(textScroller, BorderLayout.CENTER);
        TailerStopperListener stopperListener = new TailerStopperListener();
        textScroller.getVerticalScrollBar().addMouseListener(stopperListener);
        textScroller.getHorizontalScrollBar().addMouseListener(stopperListener);
        textScroller.addMouseWheelListener(stopperListener);
        textPane.addMouseListener(stopperListener);

        final TailControlAction tailControlAction = new TailControlAction(tailer, localizedResources);
        tailControl = new JButton(tailControlAction);
        tailControl.setVerticalAlignment(SwingConstants.BOTTOM);

        StampAction stampAction = new StampAction(localizedResources, document);

        configurationList = new JComboBox();

        // Build toolbar
        JToolBar toolbar = new JToolBar();
        if (additionalToolbarComponents != null) {
            for (Component component : additionalToolbarComponents) {
                toolbar.add(component);
            }
        }
        toolbar.add(new JButton(stampAction));
        toolbar.add(tailControl);
        toolbar.add(new JLabel(
                localizedResources.getMessage("nu.zoom.catonine.configurations.select")));
        toolbar.add(configurationList);
        toolbar.add(Box.createHorizontalGlue());
        tailerMessageLabel = new JLabel(tailerWaitingIcon);
        toolbar.add(tailerMessageLabel);

        add(toolbar, BorderLayout.NORTH);

        ToggleToolbarAction toggleToolbarAction =
                new ToggleToolbarAction(toolbar, localizedResources);
        // Build popup menu
        tailMenu = new JPopupMenu(
                localizedResources.getMessage("nu.zoom.catonine.tail.popuplabel"));
        tailMenu.add(tailControlAction);
        tailMenu.add(stampAction);
        tailMenu.add(toggleToolbarAction);

        tailer.addListener(document);
        tailer.addListener(this);
        tailer.addListener(tailControlAction);

        // Register keybindings
        registerKeyboardAction(
                tailControlAction,
                KeyStroke.getKeyStroke(KeyEvent.VK_F, InputEvent.SHIFT_DOWN_MASK),
                WHEN_IN_FOCUSED_WINDOW);
        registerKeyboardAction(
                stampAction,
                KeyStroke.getKeyStroke(KeyEvent.VK_T, InputEvent.CTRL_DOWN_MASK),
                WHEN_IN_FOCUSED_WINDOW);
        registerKeyboardAction(
                toggleToolbarAction,
                KeyStroke.getKeyStroke(KeyEvent.VK_L, InputEvent.CTRL_DOWN_MASK),
                WHEN_IN_FOCUSED_WINDOW);

    }

    public void setStyleRulesManager(StyleRulesManager newManager) {
        manager = newManager;
        listModel = new StyleRuleModel(manager);
        configurationList.setModel(listModel);
    }

    public synchronized void setPreferredConfiguration(final String uuid) {
        if (uuid == null) {
            throw new NullPointerException("UUID can not be null");
        }
        List<StyleRules> styleRulesList = manager.getStyleRules();
        for (StyleRules styleRules : styleRulesList) {
            if (uuid.equals(styleRules.getGuid())) {
                // Set selected style rules
                listModel.setSelectedItem(styleRules);
            }
        }
    }

    public void stop() throws IOException {
        if (tailer != null) {
            tailer.stop();
        }
    }

    public synchronized void setFile(final File fileToTail)
            throws IllegalArgumentException, FileNotFoundException, IOException {
        tailer.setFile(fileToTail);
        tailer.start();
    }

    class StyleRuleModel extends StyleRulesListModel {

        private static final long serialVersionUID = 1L;

        public StyleRuleModel(StyleRulesManager manager) {
            super(manager, true);
        }

        @Override
        public void setSelectedItem(Object anObject) {
            super.setSelectedItem(anObject);
            if (selectedConfiguration != null) {
                selectedConfiguration.removeStyleRuleListener(TailPane.this);
                selectedConfiguration.removeListener(TailPane.this);
            }
            if (anObject instanceof StyleRules) {
                selectedConfiguration = (StyleRules) anObject;
                selectedConfiguration.addStyleRuleListener(TailPane.this);
                selectedConfiguration.addListener(TailPane.this);
                try {
                    PreferencesUtil.setPreferredUUIDForFile(preferences, tailer.getFile(), selectedConfiguration.getGuid());
                } catch (InvalidDataTypeException ex) {
                    // Ignore
                } catch (BackendException ex) {
                    // Ignore
                } catch (IOException ex) {
                    // Ignore
                }
            } else {
                selectedConfiguration = null;
                try {
                    PreferencesUtil.setPreferredUUIDForFile(preferences, tailer.getFile(), null);
                } catch (InvalidDataTypeException ex) {
                    // Ignore
                } catch (BackendException ex) {
                    // Ignore
                } catch (IOException ex) {
                    // Ignore
                }
            }
            document.setConfiguration(selectedConfiguration);
            updateTailerWithListData();
            restyle();
        }
    }

    @Override
    public void listStructureChanged(StyleRules list) {
        list.removeStyleRuleListener(this);
        list.addStyleRuleListener(this);
    }

    @Override
    public void listChangedName(final StyleRules list) {
        // This is handled by the configurationLists style rule model
    }

    @Override
    public void patternChanged(StyleRule pattern) {
        restyle();
    }

    private void updateTailerWithListData() {
        if (selectedConfiguration != null) {
            tailer.setBlockMatcherStarting(selectedConfiguration.isLogBlockPatternBeginsBlock());
            tailer.setCharSet(selectedConfiguration.getFileEncoding());
            tailer.setLogBlockRegularExpression(selectedConfiguration.getLogBlockPattern());
            tailer.setReadBufferSize(selectedConfiguration.getBufferSize());
        } else {
            tailer.reset();
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see nu.zoom.catonine.StyleRulesListener#listDataChanged(nu.zoom.catonine.StyleRules)
     */
    @Override
    public void listDataChanged(StyleRules list) {
        updateTailerWithListData();
    }

    private void restyle() {
        Thread restylingThread = new Thread(new Runnable() {

            @Override
            public void run() {
                document.restyle();
            }
        });
        restylingThread.start();
    }

    @Override
    public void lines(List<LogEntry> lines) {
    }

    private void busy() {
        tailerMessageLabel.setIcon(tailerBusyIcon);
        setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        tailerMessageLabel.repaint();
    }

    @Override
    public void tailerWorking() {
        EventQueue.invokeLater(new Runnable() {

            @Override
            public void run() {
                busy();
            }
        });
    }

    private void stopped() {
        tailerMessageLabel.setIcon(tailerStoppedIcon);
        setCursor(Cursor.getDefaultCursor());
        tailerMessageLabel.repaint();
    }

    @Override
    public void tailerStopped() {
        EventQueue.invokeLater(new Runnable() {

            @Override
            public void run() {
                stopped();
            }
        });
    }

    private void waiting() {
        tailerMessageLabel.setIcon(tailerWaitingIcon);
        setCursor(Cursor.getDefaultCursor());
        tailerMessageLabel.repaint();
    }

    @Override
    public void tailerIdle() {
        EventQueue.invokeLater(new Runnable() {

            @Override
            public void run() {
                waiting();
            }
        });
    }

    class TailerStopperListener implements MouseListener, MouseWheelListener {

        @Override
        public void mouseClicked(MouseEvent e) {
        }

        @Override
        public void mouseEntered(MouseEvent e) {
        }

        @Override
        public void mouseExited(MouseEvent e) {
        }

        @Override
        public void mousePressed(MouseEvent e) {
            Component source = (Component) e.getSource();
            if (e.isPopupTrigger()) {
                tailMenu.show(source, e.getX(), e.getY());
            } else {
                stopTailer();
            }
        }

        @Override
        public void mouseReleased(MouseEvent e) {
        }

        @Override
        public void mouseWheelMoved(MouseWheelEvent e) {
            stopTailer();
        }

        private void stopTailer() {
            tailControl.setSelected(false);
            tailer.stop();
        }
    }
}
